/// Josh's Logger
/// A Redacted Software Product
/// Developed & Maintained by Josh O'Leary
/// This work is dedicated to the public domain.


#pragma once

#include <format>
#include <string>
#include <Event.h>
#include <jlog/color_codes.hpp>

#ifdef __linux__
    #define FUNCTION __PRETTY_FUNCTION__
#endif

#ifdef _WIN32
    #define FUNCTION __FUNCSIG__
#endif

namespace jlog
{
    enum class severity : uint8_t
    {
        none,
        warning,
        error,
        fatal,
        verbose,
        debug,
    };

    inline severity loglevel = severity::debug; // Default log level always debug

    struct LogEntry
    {
        severity level;
        std::string content;
        std::string timestamp;
    };

    static Event<LogEntry> on_log = Event<LogEntry>();

    inline std::vector<LogEntry> log_history;

    struct token
    {
        colorcode colorCode = color_codes::FG_DEFAULT;
        std::string content = "";
        std::string delimiter = "[]";
    };

    std::string get_timestamp();
    void log_to_console(const std::string& message);
    void log_to_file(const std::string& message);

    void log(std::vector<token> tokens);

    // Generic formatters for building loggers.
    std::vector<token> trace_format(
        const std::string& func,
        const std::string& file,
        int line);

    std::vector<token> log_format(
        const std::string& severity_name,
        const std::string& message,
        const colorcode& severity_cc = color_codes::FG_WHITE);

    std::vector<token> log_detailed_format(
    const std::string& severity_name,
    const std::string& message,
    const std::string& func,
    const std::string& file,
    int line,
    const colorcode& severity_cc = color_codes::FG_WHITE);

    // Predefined generic loggers for jlog.
    std::vector<token> info_format(const std::string& message);

    std::vector<token> info_detailed_format(
        const std::string &message,
        const std::string &func,
        const std::string &file,
        int line);

    std::vector<token> warning_format(const std::string& message);

    std::vector<token> warning_detailed_format(
        const std::string &message,
        const std::string &func,
        const std::string &file,
        int line);

    std::vector<token> error_format(const std::string& message);

    std::vector<token> error_detailed_format(
        const std::string &message,
        const std::string &func,
        const std::string &file,
        int line);

    std::vector<token> fatal_format(const std::string& message);

    std::vector<token> fatal_detailed_format(
        const std::string &message,
        const std::string &func,
        const std::string &file,
        int line);

    std::vector<token> verbose_format(const std::string& message);

    std::vector<token> verbose_detailed_format(
        const std::string &message,
        const std::string &func,
        const std::string &file,
        int line);

    std::vector<token> debug_format(const std::string& message);

    std::vector<token> debug_detailed_format(
        const std::string &message,
        const std::string &func,
        const std::string &file,
        int line);
}

#define INFO(i)    if (jlog::loglevel >= jlog::severity::none) { jlog::log(jlog::info_detailed_format(i, FUNCTION, __FILE__, __LINE__)); }

#define VERBOSE(i) if (jlog::loglevel >= jlog::severity::verbose) { jlog::log(jlog::verbose_detailed_format(i, FUNCTION, __FILE__, __LINE__)); }

#define DEBUG(i)   if (jlog::loglevel >= jlog::severity::debug) { jlog::log(jlog::debug_detailed_format(i, FUNCTION, __FILE__, __LINE__)); }

#define WARNING(i) if (jlog::loglevel >= jlog::severity::warning) { jlog::log(jlog::warning_detailed_format(i, FUNCTION, __FILE__, __LINE__)); }

#define ERROR(i)   if (jlog::loglevel >= jlog::severity::error) { jlog::log(jlog::error_detailed_format(i, FUNCTION, __FILE__, __LINE__)); }

#define FATAL(i)   if (jlog::loglevel >= jlog::severity::fatal) { jlog::log(jlog::fatal_detailed_format(i, FUNCTION, __FILE__, __LINE__)); }

#define LOGLEVEL(i) jlog::loglevel = i;

